@@ -54,18 +54,28 @@ class ScreensaverManager(GObject.Object):
 
     def lock(self, msg=""):
         """
-        Initiate locking (activating first if necessary.)
+        Initiate locking (activating first if necessary.)  Return True if we were
+        already active and just need to set the lock flag (or we were already locked
+        as well).  Return False if we're not active, and need to construct a stage, etc...
         """
         if not status.Active:
-            if self.set_active(True, True, msg):
+            if self.set_active(True, msg):
                 self.stop_lock_delay()
                 if utils.user_can_lock():
                     status.Locked = True
+                return False
         else:
             if utils.user_can_lock():
                 status.Locked = True
             self.stage.set_message(msg)
 
+        # Return True to complete any invocation immediately because:
+        # - we were already active and possibly already locked
+        # - we were unable to achieve a server grab, or something else
+        #   prevents the activation from proceeding, and we don't want to
+        #   block anything...??
+        return True
+
     def unlock(self):
         """
         Initiate unlocking and deactivating
@@ -74,7 +84,7 @@ class ScreensaverManager(GObject.Object):
         status.Locked = False
         status.Awake = False
 
-    def set_active(self, active, immediate=False, msg=None):
+    def set_active(self, active, msg=None):
         """
         Activates or deactivates the screensaver.  Activation involves:
             - sending a request to Cinnamon to exit Overview or Expo -
@@ -91,16 +101,9 @@ class ScreensaverManager(GObject.Object):
                 self.cinnamon_client.exit_expo_and_overview()
                 if self.grab_helper.grab_root(False):
                     if not self.stage:
-                        if immediate:
-                            transition = 0
-                        else:
-                            transition = c.STAGE_SPAWN_TRANSITION
-
-                        self.spawn_stage(msg, transition, self.on_spawn_stage_complete)
+                        self.spawn_stage(msg, self.on_spawn_stage_complete)
                     else:
-                        transition = 0
-
-                        self.stage.transition_in(transition, self.on_spawn_stage_complete)
+                        self.stage.activate(self.on_spawn_stage_complete)
                         self.stage.set_message(msg)
                     return True
                 else:
@@ -111,7 +114,7 @@ class ScreensaverManager(GObject.Object):
                 return True
         else:
             if self.stage:
-                self.despawn_stage(c.STAGE_DESPAWN_TRANSITION, self.on_despawn_stage_complete)
+                self.despawn_stage(self.on_despawn_stage_complete)
                 status.focusWidgets = []
             self.grab_helper.release()
             return True
@@ -152,25 +155,38 @@ class ScreensaverManager(GObject.Object):
         if not status.Active:
             return
 
+        if status.Debug and not status.Awake:
+            print("manager: user activity, waking")
+
         if status.Locked and self.stage.initialize_pam():
+            if status.Debug and not status.Awake:
+                print("manager: locked, raising unlock widget")
+
             self.stage.raise_unlock_widget()
             self.grab_helper.release_mouse()
             self.stage.maybe_update_layout()
         else:
-            GObject.idle_add(self.idle_deactivate)
+            if status.Debug:
+                print("manager: not locked, queueing idle deactivation")
+
+            trackers.timer_tracker_get().add_idle("idle-deactivate",
+                                                  self.idle_deactivate)
 
     def idle_deactivate(self):
         self.set_active(False)
+
+        trackers.timer_tracker_get().cancel("idle-deactivate")
+
         return False
 
-    def spawn_stage(self, away_message, effect_time=c.STAGE_SPAWN_TRANSITION, callback=None):
+    def spawn_stage(self, away_message, callback=None):
         """
         Create the Stage and begin fading it in.  This may run quickly, in the case of
         user-initiated activation, or slowly, when the session has gone idle.
         """
         try:
             self.stage = Stage(self, away_message)
-            self.stage.transition_in(effect_time, callback)
+            self.stage.activate(callback)
         except Exception:
             print("Could not spawn screensaver stage:\n")
             traceback.print_exc()
@@ -178,16 +194,16 @@ class ScreensaverManager(GObject.Object):
             status.Active = False
             self.cancel_timers()
 
-    def despawn_stage(self, effect_time=c.STAGE_DESPAWN_TRANSITION, callback=None):
+    def despawn_stage(self, callback=None):
         """
         Begin destruction of the stage.
         """
         self.stage.cancel_unlocking()
-        self.stage.transition_out(effect_time, callback)
+        self.stage.deactivate(callback)
 
     def on_spawn_stage_complete(self):
         """
-        Called after the stage has faded in.  All user events are now
+        Called after the stage become visible.  All user events are now
         redirected to GrabHelper, our status is updated, our active timer
         is started, and emit an active-changed signal (Which is listened to
         by our ConsoleKit client if we're using it, and our own ScreensaverService.)
@@ -202,7 +218,7 @@ class ScreensaverManager(GObject.Object):
 
     def on_despawn_stage_complete(self):
         """
-        Called after the stage has faded out - the stage is destroyed, our status
+        Called after the stage has been hidden - the stage is destroyed, our status
         is updated, timer is canceled and active-changed is fired.
         """
         was_active = status.Active == True
@@ -228,7 +244,8 @@ class ScreensaverManager(GObject.Object):
         Makes a hard grab on the Stage window, all keyboard and mouse events are dispatched or eaten
         by us now.
         """
-        self.grab_helper.move_to_window(self.stage.get_window(), True)
+        if self.stage != None:
+            self.grab_helper.move_to_window(self.stage.get_window(), True)
 
     def update_stage(self):
         """
@@ -239,7 +256,10 @@ class ScreensaverManager(GObject.Object):
         if self.stage == None:
             return
 
-        self.stage.refresh()
+        if status.Debug:
+            print("manager: queuing stage refresh (login manager reported active?")
+
+        self.stage.queue_refresh_stage()
 
     def start_timers(self):
         """
@@ -258,8 +278,7 @@ class ScreensaverManager(GObject.Object):
 
     def cancel_unlock_widget(self):
         """
-        Return to sleep (not Awake) - hides the pointer and the unlock widget,
-        which also restarts plugins if necessary.
+        Return to sleep (not Awake) - hides the pointer and the unlock widget.
         """
         self.grab_stage()
         self.stage.cancel_unlocking();
@@ -268,6 +287,9 @@ class ScreensaverManager(GObject.Object):
         """
         Updates the lock status when our timer has hit its limit
         """
+        if status.Debug:
+            print("manager: locking after delay ('lock-delay')")
+
         status.Locked = True
 
         return False
@@ -325,8 +347,6 @@ class ScreensaverManager(GObject.Object):
         """
         return self.focus_nav.get_focused_widget()
 
-# Session watcher handler:
-
     def on_session_idle_changed(self, proxy, idle):
         """
         Call back for the session client - initiates a slow fade-in
@@ -334,25 +354,4 @@ class ScreensaverManager(GObject.Object):
         if idle becomes False before it has completed its animation.
         """
         if idle and not status.Active:
-            if self.grab_helper.grab_offscreen(False):
-                self.spawn_stage("", c.STAGE_IDLE_SPAWN_TRANSITION, self.on_spawn_stage_complete)
-            else:
-                print("Can't fade in screensaver, unable to grab the keyboard")
-        else:
-            if not status.Active:
-                if self.stage:
-                    self.despawn_stage(c.STAGE_IDLE_CANCEL_SPAWN_TRANSITION, self.on_despawn_stage_complete)
-
-                trackers.timer_tracker_get().start("release-grab-timeout",
-                                                   c.GRAB_RELEASE_TIMEOUT,
-                                                   self.on_release_grab_timeout)
-
-    def on_release_grab_timeout(self):
-        """
-        Releases the initial grab during idle fade-in, when idle cancels prior to the screensaver
-        becoming fully active.
-        """
-        if not status.Active:
-            self.grab_helper.release()
-
-        return False
+            self.set_active(True)
