@@ -29,8 +29,11 @@ ALLOWED_ACTIONS = [MK.MUTE,
                    MK.TOUCHPAD_ON,
                    MK.TOUCHPAD_OFF,
                    MK.SHUTDOWN,
+                   MK.SUSPEND,
+                   MK.HIBERNATE,
                    MK.SCREEN_BRIGHTNESS_UP,
                    MK.SCREEN_BRIGHTNESS_DOWN,
+                   MK.ROTATE_VIDEO,
                    MK.KEYBOARD_BRIGHTNESS_UP,
                    MK.KEYBOARD_BRIGHTNESS_DOWN,
                    MK.KEYBOARD_BRIGHTNESS_TOGGLE]
